# Internet Services Architectures

## Spring MVC - REST Services

Michał Wójcik

---

### Change in consuming web applications

Before the era of advanced personal devices (e.g. smartphone, smartwatch, etc.), a typical server application
generated HTML documents that were rendered by web browsers.

Today, the browser client is only one of many channels for accessing services on the server:

* mobile applications - devices such as smartphones, tablets, smartwatches,
* software in devices such as Smart TV, Smart Car,
* intelligent assistants: Alexa, Siri.

New client applications require programming APIs instead of HTML documents.

---

### Approach to web applications

Since the server must provide an API implementing the business logic anyway, the browser client can also use it.
Providing the same server API to all types of client applications (mobile applications, websites, etc.) makes the
application much easier to maintain.

Web browser client applications (web frontend) use JavaScript-based frameworks:

* Angular (Google),
* React (Facebook),
* Vue.js,
* Backbone.js
* ...

---

### Web services

**Web services** - client and server applications communicating with each other via the web using the HTTP protocol:

* interoperability between applications running on different platforms and frameworks,
* descriptions processed by applications,
* use of textural representation like XML or JSON,
* loosely coupled services can be combined to create complex operations.

---

### RESTful Web Services

**Basic assumptions**:

* stateless, interaction should be immune to server restart,
* application server caching services and other elements can be used to improve performance, as long as the elements
  returned by the service are not dynamically generated and can be cached,
* possible description with WADL or OpenAPI,
* the producer and the consumer must handle the same context and the content sent,
* low data overhead, ideal for devices with limited resources,
* often used in conjunction with AJAX technology.

---

### Resources

**Resources** should be arranged in a hierarchy (nouns instead of verbs):

* `api/books` - all books,
* `api/books/1` - book with index 1,
* `api/books/1/authors` – authors of the specified book,
* `api/books/1/authors/1` – author with index 1 of the specified book.

Instead of indexes, other values that uniquely describe the item can be used:

* next index,
* domain key (e.g. isbn for books),
* generated UUID.

---

### Request parameters

**Parameter types**:

* path param - parameter that is an element of the address, indicates resources,
* query param - the parameter appended to the address (after the `?` character), make the user's (client application)
  query more precise:
  * paging: `api/books?offset=10&limit=5` - download five books starting from 10.
  * filtering: `api/books?unread=true` - download only unread books,
  * sort: `api/books?sort=latest` - get books sorted by release date.

---

### HTTP methods

Resources management is done with HTTP methods:

* `PUT` - insert resource,
* `GET`- retrieve resource,
* `POST` - add resource to collection,
* `DELETE` - delete resource.

| **operation** | **resource**   | **effect**                  |
|---------------|----------------|-----------------------------|
| GET           | `api/books`    | fetch all books             |
| GET           | `api/books/1`  | fetch single specified book |
| POST          | `/api/books`   | add new book                |
| PUT           | `/api/books`   | replace collection of books |
| PUT           | `/api/books/1` | replace specified book      |
| DELETE        | `/api/books`   | delete whole collection     |
| DELETE        | `/api/books/1` | delete specified book       |

---

### Resources operations

Because POST is not idempotent (different result for subsequent calls), it should not be used in REST approach. Instead
PUT request can be used. Additionally, PATCH request for partial update can be used.

| **operation** | **resource**   | **effect**                  |
|---------------|----------------|-----------------------------|
| GET           | `api/books`    | fetch all books             |
| GET           | `api/books/1`  | fetch single specified book |
| POST          | `/api/books`   | add new book                |
| PUT           | `/api/books`   | replace collection of books |
| PUT           | `/api/books/1` | replace specified book      |
| PATCH         | `/api/books/1` | book modification           |
| PATCH         | `/api/books`   | collection modification     |
| DELETE        | `/api/books`   | delete whole collection     |
| DELETE        | `/api/books/1` | delete specified book       |

---

### REST as CRUD

| **operacja** | **REST** | **SQL** |
|--------------|----------|---------|
| create       | PUT      | INSERT  |
| read         | GET      | SELECT  |
| update       | PACH     | UPDATE  |
| delete       | DELETE   | DELETE  |

---

### Request result

Request result should contain HTTP response code.

| **request** | **resource**   | **example response code**                |
|-------------|----------------|------------------------------------------|
| GET         | `api/books`    | 200 OK                                   |
| GET         | `api/books/1`  | 200 OK                                   |
| POST        | `/api/books`   | 201 Created, (+location)                 |
| PUT         | `/api/books`   | 201 Created, 204 No Content, (+location) |
| PUT         | `/api/books/1` | 201 Created, 204 No Content, (+location) |
| PATCH       | `/api/books/1` | 204 No Content                           |
| PATCH       | `/api/books`   | 204 No Content                           |
| DELETE      | `/api/books`   | 202 Accepted, 204 No Content, 200 OK     |
| DELETE      | `/api/books/1` | 202 Accepted, 204 No Content, 200 OK     |

Additionally, 401 Unauthorized, 403 Forbidden or 404 Not Found.

---

### Tools

Popular tools for testing REST API:

* Postman - standalone application (downloaded from homepage),
* various REST clients as web browser plugins,
* SoapUI - standalone,
* Visual Studio Code REST Client plugin,
* IntelliJ IDEA HTTP Client plugin,
* ...

---

### Rest service

```java
@RestController
@RequestMapping("/shop")
public class ShopController {

    private OrderService service;

    public ShopController(OrdersService service) {
        //...
    }

    @GetMapping("/orders")
    public List<Order> listOrders(@RequestParam(required = false) String sort) {
        //...
    }

    @GetMapping("/orders/{id}")
    public ResponseEntity<Order> getOrder(@PathVariable UUID id) {
        //...
    }

    @PostMapping("/orders")
    public ResponseEntity<Void> addOrder(@Request BodyOrder order) {
        //...
    }

}
```

---

### REST service

The above configuration allows to send the following HTTP requests (assuming the server was started on the default port
8080):

* GET on `localhost:8080/shop/orders` - returns all orders,
* GET on `localhost:8080/shop/orders?sort=asc` - returns all orders sorted ascending,
* GET on `localhost:8080/shop/orders/9eaee866-6eb3-11ea-bc55-0242ac130003` - returns order with specified id,
* POST on `localhost:8080/shop/orders` - adds new order.

---

### REST service - annotations

Used annotations:

* `@RestController` - registering a class as a controller for REST services, an instance of the class will be created
  automatically,
* `@RequestMapping` - register based address for all class methods,
* `@GetMapping` - handling GET HTTP requests,
* `@PostMapping` - handling POST HTTP requests,
* `@PathVariable` - mapping path parameter to method argument,
* `@RequestParam` - mapping query parameter to method argument,
* `@RequestBody` - mapping request body (default JSON) to an object.

---

### Binding parameters and responses

```jshelllanguage
@GetMapping("/orders")
public List<Order> listOrders(@RequestParam(required = false) String sort) {
    List<Order> orders;
    if (sort != null) { //optional query param
         orders = service.findAllSorted(sort);
    } else {
        orders = service.findAll();
    }
    return orders; //HTTP 200 code with body automatically converted to JSON array
}
```

```jshelllanguage
@GetMapping("/orders/{id}")
public ResponseEntity<Order> getOrder(@PathVariable UUID id) {
    Order order=service.find(id);
    if (order == null){
        return ResponseEntity.notFound().build(); //HTTP 404 error code
    } else {
        return ResponseEntity.ok(order); //HTTP 200 code with body in JSON format
    }
}
```

Response body:

* automatically built (by default to JSON format) based on the object returned by the method,
* using the `ResponseEntity` class that allows to control all aspects of the HTTP response (code, body, headers).

---

### Helpful resources

Guides:

* Baeldung, *REST with Spring Tutorial*, <https://www.baeldung.com/rest-with-spring-series>.
* Baeldung, *Spring Data Tutorials*, <https://www.baeldung.com/category/persistence/spring-persistence/spring-data/>.

Additional information:

* <https://www.baeldung.com/spring-controller-vs-restcontroller>
* <https://www.baeldung.com/spring-request-response-body>
* <https://www.baeldung.com/spring-pathvariable>
* <https://www.baeldung.com/spring-request-param>
* <https://www.baeldung.com/spring-requestmapping>
* <https://www.baeldung.com/spring-response-status>
* <https://www.baeldung.com/spring-request-response-body>
* <https://www.baeldung.com/building-a-restful-web-service-with-spring-and-java-based-configuration>
