# Internet Services Architectures

## Microservices

Michał Wójcik

---

### Monolithic Application

Traditional monolithic approach to building applications. Most common in old (but still maintained) systems.

![Monolithic Application](./assets/img/monolithic_application.svg)

---

### Monolithic Application - details

**Monolithic application**:

* small applications are simple to develop, everything in single place,
* all teams work on the same project (maybe different modules), know the same context,
* easy to deploy (e.g.: single war file for Java web applications),
* easy scaling by running multiple instances.

Problems start when application goes **big**:

* a lot of code in one project is difficult to understand,
* high cost of introducing new project members,
* it's difficult for teams to work independently,
* the larger code base is the slower IDE is,
* long CI/CD process,
* long-term commitment to technology stack.

---

### Programming API

With increasing popularity of introducing interoperability with desktop applications between and external systems,
there was demand for programming API instead of HTML web pages.

![Monolithic Application with SOAP WS](./assets/img/soap_services.svg)

---

### Programming API - details

Adding **Programming API** to **monolith**:

* two access modules in one application must be maintained,
* even more responsibility for project team which must have knowledge about next module.
* separate teams can develop different desktop clients without knowledge of base application implementation (only
  documentation is required).

Popular implementations: SOAP Web Services, Remote Procedure Call (RPC), Remote Method Invocation (RMI).

---

### Programming API - Web API

With increasing usage of devices like smartphones, tables, smart TV etc. there was demand for lightweight programming
API.

![Monolithic Application with REST WS](./assets/img/rest_services.svg)

---

### Programming API - Web API - details

Adding **Web API** to **monolith**:

* two access modules in one application must be maintained,
* even more responsibility for project team which must have knowledge about next module.
* separate teams can develop different desktop clients without knowledge of base application implementation (only
  documentation is required).

Popular implementations: REST, GraphQL.

---

### Separated web application

If we must maintain Web API, maybe it should be the only entry point?

![Backend and frontend applications](./assets/img/external_webapp.svg)

---

### Separated web application - details

Using **external web application**:

* front-end developers as separated team,
* knowledge about whole application implementation is not required (REST API contract is important),
* back-end developers can focus only on business logic implementation not considering interactions with users.

---

### Microservices on Single Server

Can we decompose the monolith application?

![Microservices](./assets/img/microservices.svg)

---

### Microservices on Single Server - details

Using **microservices**:

* separated modules (different projects),
* different modules can be developed using different technologies,
* modules can communicate using e.g.: REST endpoints (but not only).

Using **single application server**:

* easy configuration,
* can use communication using non-public ports,
* performance issues.

Using **single database**:

* simple in use ACID (atomicity, consistency, isolation, durability) transactions,
* one database is easy to maintain,
* schema changes must be coordinated between development teams,
* number of modules using the same database can lead to performance problems (e.g.: during long running transaction
  locks).

---

### Separated Application Servers

If modules are already separated we can move them to different servers.

![Distributed microservices](./assets/img/separated_servers.svg)

---

### Separated Application Servers - details

Using **separated application servers**:

* performance by using multiple machines,
* inside private network non-public ports can be used,
* outside private network additional security mechanism can be required.

---

### Separated Databases

If application servers are separated why not database servers?

![Distributed microservices with distributed databases](./assets/img/separated_databases.svg)

---

### Separated Databases - details

Using **separated database**:

* each service can use different architecture which is best for its purposes (e.g.: SQL, NoSQL, file system storage
  etc.),
* no ACID transactions or relationships unless we use distributed database with distributed transactions:
  * not always supported by NoSQL databases.

---

### Duplicate Database Entries

What about relationships between data stored in different databases?

![Distributed microservices and data consistency](./assets/img/duplicate_tables.svg)

---

### Duplicate Database Entries - details

Some data may need to be **duplicated**:

* relationships between objects,
* store only identifiers, not whole data,
* synchronization between modules is required:
  * it can be done by sending synchronization events between modules.

---

### API Gateway

When we have a number of web services, do we need to remember them all to use?

![Microservices with Gateway](./assets/img/gateway.svg)

---

### API Gateway - details

Using **API Gateway**:

* clients do not need to locate all the services,
* there can be different instances providing API for different clients,
* data from number of services can be joined by the gateway,
* another module to maintain,
* additional communication.

---

### Producer in Spring

**Producer**:

* some beans cannot be automatically produced by Spring Context,
* some beans require complex creation,
* calling constructors inside beans constructors is against dependency injection pattern,
* using `@Bean` annotation beans can be registered in Spring Context,
* `@Bean` annotation can be used in classes annotated with `@SpringBootApplication` or `@Configuration`.

---

### Producer - example

Declare producer method:

```java
@SpringBootApplication
public class Application {

    public static void main(String[] args) {
        SpringApplication.run(SimpleUserRpgApplication.class, args);
    }

    @Bean
    public RestTemplate restTemplate() {
        return new RestTemplate();
    }

}
```

---

### Producer - injection

From this moment `RestTemplate` can be injected as any other managed bean:

```java
@Repository
public class RestRepository {

    @Autowired
    private RestTemplate restTemplate;

}
```

---

### Producer - qualifiers

Different beans configurations (different creation parameters) can be distinguished with qualifiers.

```jshelllanguage
@Bean
@Qualifier("library")
public RestTemplate restTemplate() {
    return new RestTemplateBuilder()
            .rootUri("http://localhost:8080/library")
            .build();
}
```

```jshelllanguage
@Autowired
@Qualifier("library")
private RestTemplate restTemplate;
```

---

### Consuming REST

In Spring Web REST clients can be built with `RestTemplate`.

```jshelllanguage
RestTemmplate client = new RestTemplateBuilder()
        .rootUri("http://localhost:8080/")
        .build();
```

---

### Consuming REST - exaples

GET request:

```jshelllanguage
Book book = restTemplate.getForObject("/api/books/{id}", Book.class, id);
```

```jshelllanguage
Book[] books = restTemplate.getForObject("/api/books/", Book[].class);
```

PUT request:

```jshelllanguage
restTemplate.put("/api/books/{id}", book, id);
```

POST request:

```jshelllanguage
URI uri = restTemplate.postForLocation("/api/books/", book);
```

DELETE request:

```jshelllanguage
restTemplate.delete("/api/books/{id}", id);
```

---

### Gateway

**Gateway**:

* clients (mobile, web) need to communicate with different services,
* services decomposition should be transparent for clients,
* clients should not need to know location of all distributed services,
* there should be single gateway endpoint routing requests to particular services.

---

### Spring Cloud Gateway

Spring Cloud provides Gateway implementation:

```xml
<dependency>
    <groupId>org.springframework.cloud</groupId>
    <artifactId>spring-cloud-starter-gateway</artifactId>
</dependency>
```

---

### Spring Cloud

Spring Gateway comes from Spring Cloud project and in Spring Boot projects requires dependency configuration.

```xml
  <dependencyManagement>
      <dependencies>
          <dependency>
              <groupId>org.springframework.cloud</groupId>
              <artifactId>spring-cloud-dependencies</artifactId>
              <version>${spring-cloud.version}</version>
              <type>pom</type>
              <scope>import</scope>
          </dependency>
      </dependencies>
  </dependencyManagement>
```

Spring Cloud version must be checked with Spring Boot compatiblity chart.

---

### Gateway - configuration

Gateway routing configuration is done by providing `RouteLocator` to Spring Context:

```jshelllanguage
@Bean
public RouteLocator customRouteLocator(RouteLocatorBuilder builder) {
    return builder
            .routes()
            .route("library", r -> r
                    .host("localhost:8080")
                    .and()
                    .path("/api/books", "/api/books/**")
                    .uri("http://localhost:8081"))
            .build();
}
```

---

### Helpful Sources

Useful articles:

* Chris Richardson, *Microservice Architecture*, <https://microservices.io/>.

Additional information:

* <https://www.baeldung.com/spring-bean-annotations>
* <https://www.baeldung.com/spring-resttemplate-secure-https-service>
* <https://www.baeldung.com/spring-resttemplate-uri-variables-encode>
* <https://www.baeldung.com/spring-resttemplate-exchange-postforentity-execute>
* <https://www.baeldung.com/spring-rest-template-list>
* <https://www.baeldung.com/spring-cloud-gateway>
