# Internet Services Architectures

## JavaScript - Consuming Services

Michał Wójcik

---

### JavaScript

**JavaScript** - scripting programming language:

* commonly used with web technologies,
* weak typing,
* interpreted dynamically,
* allows to manipulate DOM objects tree,
* usually executed on client side in the browser.

---

### Script location

Different methods for adding JavaScript scripts to the HTML page:

* place it inside `<script>` tag in `<head>` or `<body>` sections,
* load external file using `<script src=""></script>`.

It's recommended to separate logic (scripts) from view (HTML document).

---

### Adding scripts to HTML document

```html
<html>
    <head>
        <script src="script.js"></script>

        <script>
            <!-- Methods declaration. -->
        </script>
    </head>

    <body>
        <script>
            <!-- Script to be executed. -->
        </script>
    </body>
</html>
```

---

### Adding scripts to HTML document - defer

```html
<html>
    <head>
        <!-- Define methods and events in external script. -->
        <script defer="defer" src="script.js"></script>
    </head>

    <body>

    </body>
</html>
```

Attribute `defer` - execute script after the document has been parsed, but before firing `DOMContentLoaded` event.

---

### Adding scripts to HTML document - module

```html
<html>
    <head>
        <!-- Define methods and events in external script. -->
        <script type="module" src="script.js"></script>
    </head>

    <body>

    </body>
</html>
```

Type `module` - causes the code to be treated as a JavaScript module.

A module is a Javascript file. However unlike a normal Javascript file, a module can specify which variables and
functions can be accessed outside the module. Other sections of the module cannot be accessed. A module can also load
other modules.

---

### Modifying the DOM tree

JavaScript allows to **modify** DOM tree elements:

* fetch element using:
  * `document.getElementById('id');`
  * `document.getElementsByTagName('name');`
  * `document.getElementsByClassName('name');`
  * `document.querySelectorAll('selector')`
* change element content with `element.innerHTML = 'value';`
* manage element attribute:
  * `element.attribute = 'value';`
  * `element.hasAttribute('attribute');`
  * `element.getAttribute('attribute');`
  * `element.setAttribute('attribute', 'value')`
* change CSS styles with `element.style.key = 'value';`

---

### Modifying the DOM tree - example

```html
<p id="test"/><!-- empty paragraph -->

<script>
    let test = document.getElementById("test");
    test.innerHTML = "Hello World!";
    test.style.color = "red";
    test.align = "right";
</script>
```

---

### Creating DOM elements

JavaScript allows for dynamic modification and creation of DOM elements:

* `document.createElement("tag name")` - create new element,
* `element.appendChild(el)` - add element to another one.

```html
<div id="container"></div>
```

```javascript
let container = document.getElementById("container");

let span = document.createElement("span");
let text = document.createTextNode("woof");

span.appendChild(text);
container.appendChild(span);
```

---

### Events handling

JavaScript allows executing specified functions as reaction to DOM tree **events**:

* loading particular element,
* change of element content,
* clicking on element (e.g.: buttons),
* ...

```html
<element event="JS code"/>
```

---

### Event handling

Selected events:

* `onchange` - element change,
* `onclick` - clicking element,
* `onmouseover` - cursor moved over the element,
* `onmouseout` - cursor moved out the element,
* `onkeydown` - pressing keyboard button,
* `onload` - loading document.

---

### Event handling - example

```html
<html>
    <head>
    </head>

    <body>
        <script>
            window.onload = function (e) {
                alert('woof');
            }
        </script>
    </body>
</html>
```

---

### Event handling - better example

```html
<html>
    <head>
    </head>

    <body>
        <script>
            document.addEventListener('load', () => {

            });
        </script>
    </body>
</html>
```

Events:

* `load` - whole page has loaded, including all dependent resources such as stylesheets, scripts, iframes, and images,
* `DOMContentLoaded` - as soon as the page DOM has been loaded, without waiting for resources to finish loading.

---

### AJAX and JavaScript

**AJAX** - Asynchronous JavaScript and XML:

* updating page without reloading whole content,
* downloading data from server,
* sending data to server.

---

### AJAX and JavaScript - example

```javascript
let xhttp = new XMLHttpRequest();

xhttp.onreadystatechange = function () {
    if (this.readyState == 4 && this.status == 200) {
        let object = JSON.parse(this.responseText);
    }
};

xhttp.open("GET", "http://localhost:8080/api/wolves", true);
xhttp.send();
```

---

### AJAX and JavaScript - modern example

```javascript
fetch('http://localhost:8080/api/wolves').then(response => {
    if (response.ok) {
        return response.json();
    }
    throw new Error("Network error.");
}).then(response => {
    console.log(response);
});
```

---

### Sending POST with JavaScript

```javascript
let xhttp = new XMLHttpRequest();
let wolf = {name: "green", age: 9};

xhttp.open("POST", "http://localhost:8090/api/wolves", true);
xhttp.setRequestHeader("Content-type", "application/json");
xhttp.send(JSON.stringify(wolf));
```

---

### Sending POST with JavaScript - modern

```javascript
let init = {
    method: 'POST',
    headers: {
        'Content-Type': 'application/json'
    },
    body: JSON.stringify(wolf)
}

fetch('http://localhost:8090/api/wolves', init).then(response => {
    if (response.ok) {
        return;
    }
    throw new Error("Network error.");
}).then(() => {
    console.log('done');
});
```

---

### Sending DELETE with JavaScript

```javascript
let xhttp = new XMLHttpRequest();

xhttp.open("DELETE", "http://localhost:8090/api/wolves/0", true);
xhttp.send();
```

---

### Sending DELETE with JavaScript - modern

```javascript
let init = {
  method: 'DELETE'
}

fetch('http://localhost:8090/api/wolves/0', init).then(response => {
    if (response.ok) {
        return;
    }
    throw new Error("Network error.");
}).then(() => {
    console.log('done');
});
```

---

### SOP vs CORS

**SOP** - Same Origin Policy:

* origin (protocol, host and port) defines web application,
* different applications can not exchange content:
  * images linking,
  * sending forms to different server;
* secures against XSS (Cross-site scripting) and CSRF (Cross-site request forgery) attacks,
* default browser behavior,
* blocks AJAX requests between applications.

**CORS** - Cross-Origin Resource Sharing:

* different applications can exchange content,
* defines using headers on the server side,
* allows for:
  * JavaScript's applications to communicate with API on different servers,
  * SSO (Single Sing On) systems for a number of applications.

---

### Helpful resources

Helpful resources:

* Mozilla, *JavaScript*, <https://developer.mozilla.org/en-US/docs/Web/JavaScript>.
* w3schools, *JavaScript Tutorial*, <https://www.w3schools.com/js/>.
