# Internet Services Architectures

## TypeScript

Michał Wójcik

---

### TypeScript language

**TypeScript**:

* statically typed language transpiled into JavaScript:
  * the developer works in TypeScript, the browser receives understandable JavaScript;
* offers compatibility with the latest versions of JavaScript (ECMAScript 2020),
* possible transpilation to an older version, e.g. ECMAScript 5:
  * at the developer's choice: `tsc --target`,
  * the problem of the standard version and the version of browsers.

---

### Basic use

Tool installation:

```bash
npm install -g typescript
```

File transpilation:

```bash
tsc file.ts
```

Monitoring file changes:

```bash
tsc --watch file.ts
```

Modern IDEs offer tool integration.

Online playground <https://www.typescriptlang.org/play>

---

### Types - basic

Basic types in TypeScript.

```typescript
let done: boolean = false;

let age: number = 42;

let color: number = 0xf00dcc;

let username: string = 'ookami';

let list1: number[] = [42, 36, 28];

let list2: Array<number> = [27, 45, 19];
```

---

### Types - enums and tuples

TypeScript allows for enumerations and tuples.

```typescript
let x: [string, number] = ['ookami', 24];

enum Color {Red, Green, Blue};

let c: Color = Color.Green;
```

---

### Types - special

Variable can be defined with `any` type.

```typescript
let variable: any = 42;
variable = 'ookami';
variable = false;
```

Special types:

* `null` - conveys that variable is empty,
* `undefined` - conveys that variable does not exist.

In non-strict mode every variable can be `null` or `undefined`. In strict mode `null` and `undefined` are treated as
separate types.

```typescript
let variable: string = '';
variable = undefined;       //error
variable = null;            //error
```

```typescript
let variable: string|undefined = '';
variable = undefined; 
variable = null;            //error
```

```typescript
let variable: string|undefined|null = '';
variable = undefined; 
variable = null;           
```

---

### Function parameters

Strong typed function parameters.

```typescript
function greeter(username: string): string {
    return 'Hello, ' + username;
}
```

Optional function parameters (always after required parameters).

```typescript
function greeter(username?: string): string {
    if (username) {
        return 'Hello, ' + username;
    } else {
        return 'Hello!';
    }
}
```

Default function parameters, anywhere (can be skipped with `undefined`).

```typescript
function greeter(user: string = 'world'): string {
    return 'Hello, ' + user;
}
```

---

### Variable number of arguments

```typescript
function greeter(firstName: string, ...otherNames: string[]): string {
    return 'Hello, ' + username + ' ' + otherNames.join(', ');
}

greeter('world', 'ookami', 'kitsune');
```

---

### Classes

```typescript
class Greeter {

    greeting: string;

    constructor(message: string) {
        this.greeting = message;
    }

    greet(): string {
        return 'Hello, ' + this.greeting;
    }

}

let greeter = new Greeter('world');
let greetings = greeter.greet();
```

---

### TypeScript - classes

```typescript
class Animal {

    name: string;

    constructor(name: string) {
        this.name = name;
    }

    move(distanceInMeters: number) {
        console.log(`${this.name} moved ${distanceInMeters}m.`);
    }

}
```

```typescript
class Snake extends Animal {

    constructor(name: string) {
        super(name);
    }

    move(distanceInMeters = 5) {
        console.log('Slithering...');
        super.move(distanceInMeters);
    }

}
```

---

### TypeScript - visibility levels

**Visibility levels**:

* public - default, fields and methods visible in other classes,
* protected - visible only in inheritance hierarchy,
* private - visible only in class.

```typescript
class Animal {

    private _name: string = '';

    get name(): string {
        return this._name;
    }

    set name(name: string) {
        this._name = name;
    }

}
```

---

### TypeScript - abstract classes

```typescript
abstract class Animal {

    abstract makeSound(): void;

    move(): void {
        console.log("move");
    }

}
```

---

### TypeScript - interfaces

```typescript
interface Moveable {

    move(distanceInMeters: number): void;
    
}
```

```typescript
class Snake implements Moveable {
    
    move(distanceInMeters = 5) {
        console.log("Slithering...");
    }
    
}
```

---

### TypeScript - interfaces with properties

```typescript
interface Person {

    firstName: string;

    lastName: string;

    get email(): string;

}
```

```typescript
class Student implements Person {

    firstName: string;

    lastName: string;

    constructor(firstName: string, lastName: string) {
        this.firstName = firstName;
        this.lastName = lastName;
    }

    get email(): string {
        return this.firstName
            + "." + this.lastName
            + "@student.pg.edu.pl";
    }

}
```

```typescript
let p1: Person = new Student('Wolf', 'Bad');
```

---

### Helpful resources

Helpful resources:

* Microsoft, *TypeScript Documentation*, <https://www.typescriptlang.org/docs/>.
* w3schools, *TypeSript Tutorial*, <https://www.w3schools.com/typescript/>.
