# Internet Services Architectures

## Microservices - Discovery

Michał Wójcik

---

### Services location

**Service provider**:

* same service can be deployed at different localizations,
* localization can change dynamically (e.g.: cluster nodes migration),
* there can be multiple instance of the same service.

**Service consumer**:

* needs to know service provider localization,
* needs to be aware of localization changes,
* needs to choose which instance use.

---

### Discovery

**Discovery**:

* services can be deployed on different addresses,
* services should not need to know exactly where other services are,
* there should be single (or distributed) catalog service,
* all services need to know only address of the catalog service.

---

### Discovery - solutions

There are multiple solutions for discovery services:

* **Consul** by HashiCorp - standalone catalog with service discovery and health checking,
* **Eureka** by Netflix - discovery server integrated into Spring Boot applications,
* **Zookeeper** by Apache - distributed key value store which can be used for service discovery,
* **etcd** - distributed key value store which can be used for service discovery,
* **Kubernetes** - container orchestration solution with built-in service discovery.

---

### Consul - server

Consul can be started with Docker command:

```bash
docker run --rm --name consul -p 8500:8500 consul:1.10.3
```

---

### Consul - client

In client module appropriate dependency is required:

```xml
<dependency>
    <groupId>org.springframework.cloud</groupId>
    <artifactId>spring-cloud-starter-consul-all</artifactId>
</dependency>
```

and configuration in `application.properties`:

```properties
spring.cloud.consul.host=localhost
spring.cloud.consul.port=8500
spring.cloud.consul.discovery.instance-id=1
spring.cloud.consul.discovery.service-name=library
```

---

### Eureka - server

Eureka can be started as Spring Boot application:

```java
@SpringBootApplication
@EnableEurekaServer
public class EurekaApplication {
    
    public static void main(String[] args) {
        SpringApplication.run(EurekaApplication.class, args);
    }

}
```

```xml
<dependency>
    <groupId>org.springframework.cloud</groupId>
    <artifactId>spring-cloud-starter-netflix-eureka-server</artifactId>
</dependency>
```

---

### Eureka - client

In client module appropriate dependency is required:

```xml
<dependency>
    <groupId>org.springframework.cloud</groupId>
    <artifactId>spring-cloud-starter-netflix-eureka-client</artifactId>
</dependency>

```

and configuration in `application.properties`:

```properties
eureka.client.service-url.defaultZone=http://localhost:8761/eureka
eureka.instance.appname=Library
eureka.instance.instance-id=1
eureka.instance.lease-expiration-duration-in-seconds=5
eureka.instance.lease-renewal-interval-in-seconds=2
```

---

### Discovery client - configuration

Spring discovery client is independent of implementation (Consul, Eureka).

Enable client on main class (not required in newer versions):

```java
@SpringBootApplication
@EnableDiscoveryClient
public class Application {

    public static void main(String[] args) {
        SpringApplication.run(Application.class, args);
    }

}
```

---

### Discovery client - use

Inject discovery client into component:

```java
@Repository
public class RestRepository {

    @Autowired
    private DiscoveryClient discoveryClient;

    @Autowired
    private RestTemplate restTemplate;

    public void delete() {
            URI uri = discoveryClient.getInstances("library")
                .stream()
                .findAny()
                .orElseThrow()
                .getUri();
            restTemplate.delete(uri + "/" + id);
    }
}
```

---

### Helpful resources

Helpful resources:

* <https://www.baeldung.com/cs/service-discovery-microservices>
* <https://www.baeldung.com/spring-cloud-consul>
* <https://www.baeldung.com/spring-cloud-netflix-eureka>
* <https://www.baeldung.com/eureka-self-preservation-renewal>
* <https://www.baeldung.com/spring-cloud-custom-gateway-filters#bd-1-checking-and-modifying-the-request>
