# Internet Services Architectures

## Database Migration

Michał Wójcik

---

### Database structure

From where comes **database structure**:

* it is already present (new project for existing database),
* developed together with the project,
* developed independently of the project.

---

### Database structure - create

How to create database structure:

* write SQL scripts:
  * can be executed independently,
  * can be integrated into container configuration;
* use JPA auto generation feature:
  * can be created automatically on application startup.

---

### Database structure - develop

Database structures are not static:

* can change (quite often) during project rapid development before first release:
  * no need to maintain production data;
* can change (probably rarely) during project maintenance:
  * need to maintain production data;
* can change during project development (introducing new features, etc.) after releases:
  * need to maintain production data.

---

### Database structure - develop scenarios

What to do with existing data:

* let JPA make migration:
  * executed automatically on application startup,
  * will work for simple changes like adding new column;
* write migration SQL:
  * can deal with complex changes,
  * executed independently;
* use migration framework:
  * executed automatically on application startup,
  * can be executed independently using CLI,
  * can deal with complex changes,
  * can be written in SQL or framework notation.

---

### Migration tools

Database migration tools:

* Flyway - supports version control for database changes using plain SQL scripts or Java code using JDBC,
* Liquibase - uses XML, YAML, or JSON to define database changes and supports various databases,
* Migrate - a Go based tool based on SQL scripts,
* Alembic - a Python SQL toolkit with script based migrations,
* db-migrate - Node.js applications migration tool with script based migrations.

---

### Liquibase

Dependency for Liquibase:

```xml
<dependency>
    <groupId>org.liquibase</groupId>
    <artifactId>liquibase-core</artifactId>
</dependency>
```

Changelog (migrations) can be provided in XML and need to be configured in `application.properties`:

```properties
spring.liquibase.change-log=classpath:/db/changelog.xml
```

---

### Liquibase - changelog

Main changelog can include number of changelogs:

```xml
<databaseChangeLog
        xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
        xmlns="http://www.liquibase.org/xml/ns/dbchangelog"
        xsi:schemaLocation="http://www.liquibase.org/xml/ns/dbchangelog
         http://www.liquibase.org/xml/ns/dbchangelog/dbchangelog-3.1.xsd">

    <include file="changelog/01-create-scheme.xml"
             relativeToChangelogFile="true"/>

</databaseChangeLog>
```

---

### Liquibase - migration

Migration described in XML format:

```xml
<databaseChangeLog
        xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
        xmlns="http://www.liquibase.org/xml/ns/dbchangelog"
        xsi:schemaLocation="http://www.liquibase.org/xml/ns/dbchangelog
         http://www.liquibase.org/xml/ns/dbchangelog/dbchangelog-3.1.xsd">

    <changeSet id="01" author="psysiu">

        <createTable tableName="books">
            <column name="isbn" type="varchar(255)">
                <constraints primaryKey="true"/>
            </column>
            <column name="title" type="varchar(255)"/>
        </createTable>
    </changeSet>

</databaseChangeLog>
```

---

### Liquibase - migrations

Changeset entries:

* **Create Table** (`createTable`): Defines a new table schema in the database.
* **Drop Table** (`dropTable`): Removes an existing table from the database.
* **Add Column** (`addColumn`): Adds a new column to an existing table.
* **Drop Column** (`dropColumn`): Removes a column from an existing table.
* **Rename Column** (`renameColumn`): Renames a column in an existing table.
* **Add Unique Constraint** (`addUniqueConstraint`): Adds a unique constraint to one or more columns in a table.
* **Drop Unique Constraint** (`dropUniqueConstraint`): Removes a unique constraint from one or more columns in a table.
* **Add Foreign Key Constraint** (`addForeignKeyConstraint`): Creates a foreign key constraint between tables.
* **Drop Foreign Key Constraint** (`dropForeignKeyConstraint`): Removes a foreign key constraint between tables.
* **SQL** (`sql`): Executes arbitrary SQL statements.
* **Include** (`include`): References and includes an external changelog file.
* **Custom Change** (`customChange`): Executes custom Java logic as a change.

---

### Liquibase - different DB

What if developer is using different DB server in tests (e.g. H2) and different DB server on production (e.g. MySQL or
PostgreSQL).

```xml
<?xml version="1.1" encoding="UTF-8" standalone="no"?>
<databaseChangeLog
        xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
        xmlns="http://www.liquibase.org/xml/ns/dbchangelog"
        xsi:schemaLocation="http://www.liquibase.org/xml/ns/dbchangelog
         http://www.liquibase.org/xml/ns/dbchangelog/dbchangelog-3.1.xsd">
  
    <changeSet id="5" author="psysiu" dbms="postgresql">
        <addColumn tableName="files" >
            <column name="content" type="BYTEA"/>
        </addColumn>
    </changeSet>

    <changeSet id="5" author="psysiu" dbms="h2">
        <addColumn tableName="files" >
            <column name="content" type="BLOB"/>
        </addColumn>
    </changeSet>

</databaseChangeLog>
```

---

### Helpful resources

Helpful resources:

* Liquibase Inc. *Liquibase Documentation* <https://docs.liquibase.com/home.html>
* <https://www.baeldung.com/liquibase-refactor-schema-of-java-app>
* <https://www.baeldung.com/liquibase-rollback>
* <https://www.baeldung.com/liquibase-vs-flyway>
* <https://www.baeldung.com/database-migrations-with-flyway>
