# Internet Services Architectures

## Messaging

Michał Wójcik

---

### Messaging in distributed systems

**Messaging**:

* loosely coupled communication:
  * no direct connection between sender and receiver,
  * senders and receivers can connect at any time,
  * everyone knows localization of messaging server;
* communication based on messages,
* asynchronous communication.

---

### Messages destinations

When talking about messaging, there is usually common approach for message destinations:

* Queue:
  * p2p model,
  * only one consume for message,
  * no time dependencies,
  * confirmation of receipt required.
* Topic:
  * publish/subscribe model,
  * the possibility of having multiple consumers per message,
  * time dependencies,
  * confirmation of receipt not required.

---

### Messaging systems

In Java, there are two populary used messaging approaches:

* JMS:
  * Java Messaging Service,
  * use serialization mechanisms,
  * part of Jakarta EE specification,
  * designed for Java frameworks.
* AMQP:
  * The Advanced Message Queuing Protocol,
  * platform-neutral,
  * binary application level protocol.

---

### Messaging systems - implementations

Each of approaches have multiple implementations:

* JMS:
  * RabbitMQ (client plugin).
  * Apache ActiveMQ,
  * Apache ActiveMQ Artemis,
  * OpenMQ.
* AMQP:
  * RabbitMQ,
  * Apache ActiveMQ,
  * Apache ActiveMQ Artemis,
  * Apache Qpid,
  * Microsoft Azure Service Bus.

---

### Other messaging systems

Other messaging systems:

* Apache Kafka,
* Apache Pulsar,
* RSocket,
* ...

---

### RabbitMQ

RabbitMQ can be started as Docker container.

```yaml
services:
  rabbitmq:
    image: rabbitmq:3.12.9-management-alpine
    restart: always
    ports:
      - "5672:5672"
      - "15672:15672"
```

---

### Dependency

For dependency, general AMQP dependency can be used.

```xml
<dependency>
    <groupId>org.springframework.boot</groupId>
    <artifactId>spring-boot-starter-amqp</artifactId>
</dependency>
```

This project provides support for using Spring and Java with AMQP 0.9.1, and in particular RabbitMQ.

---

### Defining message goals

Message goal can be configured in the broker server, but if application has admin rights, those can be automatically
created on application startup.

```jshelllanguage
@Bean
public Queue professionQueue() {
    return new Queue("libraryQueue", true);
}
```

```jshelllanguage
@Bean
public TopicExchange libraryTopic() {
    return new TopicExchange("libraryTopic");
}
```

---

### Sending message

```java
@Repository
public class MessageRepository {

    @Autowired
    private final RabbitTemplate rabbitTemplate;

    public void notifyOne(Book book) {
        rabbitTemplate.convertAndSend("libraryQueue", book);
    }

    public void NotifyAll(Book book) {
        rabbitTemplate.convertAndSend("libraryTopic", book);
    }

}
```

Sending messages requires broker implementation object, like `RabbitTemplate`.

---

### Receiving messages

```java
@Component
public class ProfessionRabbitListener {

    @RabbitListener(queues = "bookQueue")
    public void onMessage(Book book) {
        
    }

}
```

Receiving messages requires broker implementation annotation, like `@RabbitListener`.

---

### Different message types

```java
@Component
@RabbitListener(queues = "bookQueue")
public class ProfessionRabbitListener {

    @RabbitHandler
    public void onDeleteMessage(DeleteBook book) {
        
    }

    @RabbitHandler
    public void onCreateMessage(CreateBook book) {
  
    }

}
```

---

### Helpful resources

Helpful resources:

* <https://www.rabbitmq.com/documentation.html>
* <https://www.baeldung.com/rabbitmq>
* <https://www.baeldung.com/spring-amqp>
* <https://docs.spring.io/spring-boot/docs/current/reference/html/messaging.html>
