# Internet Services Architectures

## Open Authorization

Michał Wójcik

---

### Authentication vs authorization

**Authentication** - process of verifying the identity of a user or system attempting to access a resource. It involves
confirming that the credentials provided (such as username/password, tokens, biometrics, etc.) match the expected
credentials stored in a system.

**Authorization** - process of determining what actions or resources a verified and authenticated user or system is
allowed to access or perform within the application or system. It involves checking the permissions, roles, or
privileges associated with the authenticated identity and granting or denying access based on those permissions.

---

### User authentication

User authentication:

* Basic Authentication - setting username and password encoded with Base64 sent in each request using `Authorization`
  header,
* Form Authentication - user credentials collected using form and appropriate session object is created.
* API key - user is authorized on a basic of API key added to each request (in header or query param).

Those are still widely used but quite legacy methods.

---

### Keeping session

How to keep session in subsequent requests:

* Cookie-based Authentication - authenticated user's session is identified by cookie with session ID sent with each
  request, cookie is set in response (e.g. after form authentication),
* Query param Authentication - authenticated user's session is identified by session ID passed with each.
* Token - authorized user is identified by token created during authentication process.

Those are still widely used. For example JSF by default now uses cookie but in past used query param by default.

---

### OAuth

**OAuth** (Open Authorization) - authorization protocol allowing third-party applications to access user data without
exposing credentials. It works by providing tokens (access tokens) to the third-party app after user consent, allowing
restricted access to specific resources.

**OAuth2** - updated version of OAuth with multiple flows designed for different scenarios.

---

### OAuth roles

Roles in OAuth:

* **Resource Owner:** The entity that can grant access to a protected resource. Typically, this is the end-user.
* **Client:** An application requesting access to a protected resource on behalf of the resource owner. This could be a
  web or mobile application.
* **Resource Server:** The server hosting the protected resources that can only be accessed with proper authorization.
* **Authorization Server:** The server responsible for authenticating the resource owner and providing access tokens to
  the client after successful authentication.

---

### OAuth2 flows

OAuth2 flows:

* **Client Credentials Flow** - Utilized by confidential clients (such as servers) to authenticate and obtain access
  tokens based on client credentials (not user credentials).
* **Resource Owner Password Credentials Flow** - Allows the client to directly obtain user credentials (
  username/password) and exchange them for an access token. Less secure and should be used cautiously.
* **Authorization Code Flow** - A flow used by web and mobile apps where the client requests authorization from the
  resource owner through a redirection process. It exchanges an authorization code for an access token.
* **Implicit Flow** - Primarily used by single-page web apps (SPAs) where access tokens are issued directly without
  exchanging an authorization code. It's less secure due to tokens being exposed in the browser.
* **Device Authorization Flow** - Suitable for devices with limited input capabilities (e.g., smart TVs) where users can
  authenticate via a secondary device, obtaining an access token.

---

### Client Credentials Flow

![Client Credentials Flow](./assets/img/client-redentials-flow.svg)

---

### Resource Owner Password Credentials

![Resource Owner Password Credentials](./assets/img/resource-owner-password-flow.svg)

---

### Authorization Code Flow

![Authorization Code Flow](./assets/img/authorization-code-flow.svg)

---

### Authorization Code Flow with PKCE

![Authorization Code Flow with PKCE](./assets/img/authorization-code-flow-with-proof-key-for-code-exchange-pkce.svg)

---

### Device Authorization Flow

![Device Authorization Flow](./assets/img/device-authorization-flow.svg)

---

### Authorization server

So well known authorization servers with OAuth2 support:

* Keycloak,
* Auth0,
* Spring Authorization Server,
* Okta,
* Azure Active Directory,
* Google Identity Platform,
* ...

---

### Spring authorization server

Spring Authorization Server - ready to use authorization server with OAuth2 support.

```xml
<dependency>
    <groupId>org.springframework.security</groupId>
    <artifactId>spring-security-oauth2-authorization-server</artifactId>
</dependency>
```

---

### Spring authorization server - clients

Whole configuration can be made with properties with almost no code using `spring.security.oauth2.authorizationserver`
properties defining clients.

```properties
m2m.registration.client-id=m2m-client
m2m.registration.client-secret={noop}m2m-secret
m2m.registration.authorization-grant-types[0]=client_credentials
m2m.registration.client-authentication-methods[0]=client_secret_basic
m2m.registration.client-authentication-methods[1]=client_secret_post
```

---

### Spring authorization server - users

Users can be defined with `UserDetailsService`. For easy development and testing it can be done in-memory.

```jshelllanguage
@Bean
public UserDetailsService users() {
    return new InMemoryUserDetailsManager(
            User.withUsername("admin")
                    .password("{noop}adminadmin")
                    .build()
    );
}
```

---

### Spring resource server

Resource server in Spring can be configured with **Spring Security** and **OAuth2 Resource Server**.

```xml
<dependencies>
    <dependency>
        <groupId>org.springframework.boot</groupId>
        <artifactId>spring-boot-starter-security</artifactId>
    </dependency>

    <dependency>
        <groupId>org.springframework.boot</groupId>
        <artifactId>spring-boot-starter-oauth2-resource-server</artifactId>
    </dependency>
</dependencies>
```

For simple configuration it is enoguh to present `jww.issuer-uri` property.

```properties
spring.security.oauth2.resourceserver.jwt.issuer-uri=http://localhost:8090
```

---

### Helpful resources

OAuth2 flows:

* <https://auth0.com/docs/get-started/authentication-and-authorization-flow/which-oauth-2-0-flow-should-i-use>
* <https://auth0.com/docs/get-started/authentication-and-authorization-flow/client-credentials-flow>
* <https://auth0.com/docs/get-started/authentication-and-authorization-flow/resource-owner-password-flow>
* <https://auth0.com/docs/get-started/authentication-and-authorization-flow/authorization-code-flow>
* <https://auth0.com/docs/get-started/authentication-and-authorization-flow/authorization-code-flow-with-proof-key-for-code-exchange-pkce>
* <https://auth0.com/docs/get-started/authentication-and-authorization-flow/device-authorization-flow>

Helpful resources:

* <https://www.baeldung.com/spring-security-5-default-password-encoder>
* <https://www.baeldung.com/spring-security-enable-logging>
* <https://docs.spring.io/spring-authorization-server/reference/getting-started.html>
* <https://docs.spring.io/spring-security/reference/servlet/oauth2/resource-server/jwt.html>

---

### Helpful resources - continued

Standards:

* <https://datatracker.ietf.org/doc/html/rfc6749>
* <https://datatracker.ietf.org/doc/html/draft-ietf-oauth-v2-1-09>

Utils:

* <https://jwt.io/>
* <https://www.base64encode.org/>
* <https://tonyxu-io.github.io/pkce-generator/>
