# Internet Services Architectures

## Docker

Michał Wójcik

---

### Motivation

Why deploying applications is troublesome:

* application needs to be configured (e.g. database connection, file storage, ports, etc.),
* application requires runtime environment:
  * Java Virtual Machine for Java based applications,
  * Application Server for web applications,
  * HTTP server for static web applications;
* runtime environment must be configured,
* additional system libraries can be required.

Why developing applications is troublesome:

* different database severs or version for different projects,
* projects build with multiple modules required to be running for development,
* projects using legacy tools unable to be installed in developer's operating system.

---

### Containers

**Virtual Machine** - virtualize an entire machine down to the hardware layers and containers only virtualize software
layers above the operating system level.

**Containers**:

* in some way similar to virtual machines,
* lightweight software packages that contain all the dependencies required to execute the contained software
  application:
  * system libraries,
  * external third-party code packages,
  * other operating system level applications.

---

### Containers vs Virtual Machines

Containers vs Virtual machines:

* **Containers:**
  * kernel shared with the host,
  * no hardware isolation,
  * small sizes,
  * containers based on the same image share its content,
  * very small performance overhead,
  * can run within minutes;
* **Virtual Machines:**
  * separate system,
  * hi level hardware separation (Intel VT-x, AMD-V),
  * big size,
  * performance problems,
  * can take some time to start.

---

### Containers vs Virtual Machines - stack

![Containers vs Virtual Machines](./assets/img/containers vs vms.svg)

---

### Containers vs Virtual Machines - P&C

**Containers**:

* **pros:**
  * fast to modify and iterate on,
  * pre-made containers in public repositories;
* **cons:**
  * shared host exploits.

**Virtual Machines**:

* **pros:**
  * full isolation security,
  * interactive development;
* **cons:**
  * iteration speed,
  * storage size cost.

---

### Containers - usage

Usage for **containers**:

* production environment,
* development environment,
* microservices,
* applications abstraction from physical infrastructure,
* solution for incompatible requirements and dependencies.

---

### Docker software

**Docker** - example for container platform:

* allows to create images,
* allows to run containers based on created images,
* containers run applications,
* allows to create networks, so containers can communicate.

---

### Container

**Container**:

* encapsulated environment that runs applications,
* lightweight alternative for virtual machine,
* doesn't have own kernel – uses hosts kernel,
* containers can communicate with each others via network,
* has its own filesystem,
* has its own set of ports which can be published in host.

---

### Image

**Image**:

* contains application and all dependencies/resources required,
* ready to run application independently of operating system, filesystem, etc.,

**Image**:

* **pros**:
  * portable,
  * light in comparison to virtualization,
  * extendable,
  * easy to configure and run,
  * readonly;
* **cons**:
  * size in comparison to native application,
  * more resources use in comparison to native applications.

---

### Docker CLI

**Docker CLI** provides hierarchical commands.

`docker` - main command:

```bash
docker
```

`pull` - pull image from repository:

```bash
docker pull hello-world
```

`run` - runs container:

```bash
docker run hello-world
```

```bash
docker run nginx
```

---

### Docker CLI - images

`image` - images management:

```bash
docker image
```

`pull` - pull image from repository:

```bash
docker image pull hello-world
```

`ls` - list images:

```bash
docker image ls
```

`rm` - remove image:

```bash
docker image rm hello-world  
```

---

### Docker CLI - volumes

`volume` - volumes management:

```bash
docker volume
```

`ls` - list volumes:

```bash
docker volume ls
```

`create` - create a volume:

```bash
docker volume create woof
```

`rm` - remove volume:

```bash
docker volume rm woof
```

`prune` - remove unused volumes:

```bash
docker volume prune
```

---

### Docker CLI - containers

`container` - containers management:

```bash
docker container
```

`run` - runs container:

```bash
docker container run hello-world
```

`ls` - list containers:

```bash
docker container ls
```

`attach` - attach input, output and error streams:

```bash
docker container attach laughing_wright
```

---

### Docker CLI - containers - continued

`stop` - stop container:

```bash
docker container stop laughing_wright
```

`start` - start container:

```bash
docker container start laughing_wright
```

`rm` - remove container:

```bash
docker container rm laughing_wright
```

`prune` - remove all stopped containers:

```bash
docker container prune
```

---

### Running a container

Running container:

* can be removed when finished,
* can be started in background,
* can be named,
* it's good to provide exact version, `lates` can surprise.

```bash
docker container run --rm -d --name nginx-isa nginx:1.23.3 
```

---

### Running a container - example

Running command in container:

```bash
docker container exec nginx-isa ls /
```

Running interactive command in container

```bash
docker container exec -it nginx-isa bash
```

---

### Docker CLI - networking

`netowork` - network management:

```bash
docker network
```

`ls` - list networks:

```bash
docker network ls
```

`create` - create network:

```bash
docker network create isa-network
```

`rm` - remove network:

```bash
docker network rm isa-network
```

---

### Networking - example

```bash
docker run --rm -dit --name alpine1 alpine ash
```

```bash
docker run --rm -dit --name alpine2 alpine ash
```

```bash
docker exec alpine1 ping alpine2
```

```bash
docker network create alpine-net
```

```bash
docker run --rm -dit --name alpine3 --network alpine-net alpine ash
```

```bash
docker run --rm -dit --name alpine4 --network alpine-net alpine ash
```

```bash
docker exec alpine3 ping alpine4
```

```bash
docker stop alpine1 alpine2 alpine3 alpine4
```

```bash
docker network rm alpine-net
```

---

### Dockerfile

`Dockefile` - is a recipe how to build image:

* define base image,
* define exposed ports,
* define volumes,
* execute commands (e.g. install dependencies),
* copy files (e.g. application).

---

### Dockerfile - commands

`Dockerfile` is made with a sequence of commands:

* `FROM` - Specify the base image for your Docker image. It is the starting point for your image.
* `WORKDIR` - Set the working directory within the container for subsequent commands.
* `COPY` - Copy files or directories from your host machine into the container.
* `ADD` - Similar to `COPY`, but can also fetch files from remote URLs and unpack compressed files.
* `RUN` - Execute commands within the container during the build process. Used for installing software or configuring
  the environment.
* `CMD` - Define the default command to run when a container is started from the image. This can be overridden at
  runtime.
* `ENTRYPOINT` - Specify the default executable when the container is run. It is more commonly used for defining the
  primary command, which cannot be overridden.
* `EXPOSE` - Document which network ports the container listens on at runtime, but it doesn't actually publish the
  ports.
* `ENV` - Set environment variables within the container, which can be used by the processes running in the container.

---

### Dockerfile - commands - continued

`Dockerfile` is made with a sequence of commands:

* `ARG` - Define build-time arguments that can be passed to the Dockerfile during the `docker build` command.
* `LABEL` - Add metadata to the image, such as version information or maintainer details.
* `VOLUME` - Create a mount point for externally defined volumes to be used by the container.
* `USER` - Set the user or UID that the container process should run as, enhancing security.
* `HEALTHCHECK` - Define a command to periodically check the health of the container.

---

### Standard description

To provide unified manner for describing containers, The OpenContainers Image Spec can be used:
<https://specs.opencontainers.org/image-spec/?v=v1.0.1>

Some of the label values:

* org.opencontainers.image.title,
* org.opencontainers.image.authors,
* org.opencontainers.image.source,
* org.opencontainers.image.url,
* org.opencontainers.image.vendor,
* org.opencontainers.image.version,
* org.opencontainers.image.description,
* org.opencontainers.image.licenses.

---

### Dockerfile - example

```dockerfile
FROM nginx:1.23.3 

EXPOSE 80

VOLUME /usr/share/nginx/html/custom

RUN echo '<html>' > /usr/share/nginx/html/index.html
RUN echo '  <head>' >> /usr/share/nginx/html/index.html
RUN echo '    <link rel="stylesheet" type="text/css" href="/custom/custom.css">' \
>> /usr/share/nginx/html/index.html
RUN echo '  </head>' >> /usr/share/nginx/html/index.html
RUN echo '  <body>' >> /usr/share/nginx/html/index.html
RUN echo '    <h1>Welcome to My Custom Nginx Page</h1>' >> /usr/share/nginx/html/index.html
RUN echo '    <p>This is a custom Nginx web page.</p>' >> /usr/share/nginx/html/index.html
RUN echo '  </body>' >> /usr/share/nginx/html/index.html
RUN echo '</html>' >> /usr/share/nginx/html/index.html
```

```bash
docker build -t isa-nginx:1.0.0 .
```

```bash
docker run --rm -p 8080:80 -v $(pwd)/custom:/usr/share/nginx/html/custom isa-nginx:1.0.0
```

---

### Docker Compose

**Docker Compose** - tool for configuring and starting multiple container applications:

* configuration in `yaml` format,
* automatic network connection,
* simple commands to start/stop everything,
* wrapper for docker commands.

---

### Docker Compose - example

```yaml
version: '3'

services:
  nginx:
    image: custom-nginx:1.0.0
    ports:
      - "8080:80"
    volumes:
      - /tmp/custom:/usr/share/nginx/html/custom
```

---

### Container storage

Dynamic data should not be stored inside container (those need to be easily replaceable):

* **bind mounts**:
  * allow to mount a file or directory from the host machine directly into a container,
  * be careful about rights,
  * changes can be done from container and host;
* **named volumes**:
  * provide a named reference to a directory on the host system where the data is stored,
  * Docker-managed way of creating persistent storage,
  * data in named volumes persists even if the associated container is removed;
* **anonymous volumes**:
  * automatically created by Docker and associated with a container,
  * don't have a user-defined name and are often used for temporary or cache storage,
  * automatically deleted when the container is removed.

---

### Docker Compose - volume example

```yaml
version: '3'

services:
  mysql:
    image: mysql:latest
    environment:
      MYSQL_ROOT_PASSWORD: root_password
      MYSQL_DATABASE: database_name
      MYSQL_USER: database_user
      MYSQL_PASSWORD: database_password
    volumes:
      - mysql-data:/var/lib/mysql
    ports:
      - "3306:3306"

volumes:
  mysql-data:
```

---

### Docker Compose CLI

Selected commands:

* `docker-compose up` - creates and starts services defined in a `docker-compose.yml` file,
* `docker-compose down` - stop and remove containers and networks,
* `docker-compose ps` - list the status of services defined in the `docker-compose.yml` file,
* `docker-compose logs` - display logs for services defined in the compose file.

---

### Helpful resources

Helpful resources:

* Docker Inc., *Docker Docs*, <https://docs.docker.com/>.
* Baeldung, *Docker Guide*, <https://www.baeldung.com/ops/docker-guide>.
